package se.mdh.assist.analysis.plugin.popup.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.Calendar;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.fordiac.ide.model.libraryElement.Application;
import org.fordiac.ide.model.libraryElement.FBType;
import org.fordiac.ide.model.libraryElement.I4DIACElement;
import org.fordiac.ide.typeexport.FbtExporter;
import org.fordiac.ide.typeimport.FBTImporter;
import org.fordiac.ide.typeimport.exceptions.ReferencedTypeNotFoundException;
import org.fordiac.systemmanagement.SystemManager;

import se.mdh.assist.analysis.wcet.WCETAnalysis;
import se.mdh.assist.analysis.wcet.data.FBWCETData;
import se.mdh.assist.analysis.wcet.data.WCETAnalysisOptions;
import se.mdh.assist.analysis.wcet.gui.FBWCETPresentationDialog;
import se.mdh.assist.analysis.wcet.gui.WCETAnalysisOptionsEditorDialog;

public class WCETAnalysisPopupAction implements IObjectActionDelegate {

	protected Shell shell;
	private IFile selectedFile;
	private Application application;
	private FBType fbType;
	
	/**
	 * Constructor for Action1.
	 */
	public WCETAnalysisPopupAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		shell = targetPart.getSite().getShell();
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		if (selectedFile != null || fbType != null || application != null) {
			//TransactionalEditingDomain domain = TransactionalEditingDomain.Factory.INSTANCE.createEditingDomain();
			//domain.setID("se.mdh.assist.analysis");
			FBWCETData result = null;
			long runningTime = 0;
			try {
				
				if (selectedFile != null) {
					fbType = new FBTImporter().importFBType(selectedFile, SystemManager.getInstance().getPalette(selectedFile.getProject()));
				}
				I4DIACElement analysisTarget = (fbType != null) ? fbType : application;
				
				//WCETDataList.setNormalizationMethod(NormalizationMethod.MAXIMIZE);
				
				WCETAnalysisOptions options = new WCETAnalysisOptions();
				OptionsDialogRunner dialogRunner = new OptionsDialogRunner(options);
				SwingUtilities.invokeAndWait(dialogRunner);
				if (dialogRunner.isResultOk()) {
					options = dialogRunner.getOptions();
				} else {
					return;
				}
				System.out.println(options.toString());
				
				WCETAnalysis analysis = new WCETAnalysis(analysisTarget, options);
				long startTime = Calendar.getInstance().getTimeInMillis();
				result = analysis.doAnalysis();
				runningTime = Calendar.getInstance().getTimeInMillis() - startTime;
				if (selectedFile != null) {
					FbtExporter.saveAsFBTFile(fbType, selectedFile);
				} else if (application != null) {
					// TODO How to save?
				}
				System.out.println(result.toString());
				System.out.println("Time: "+runningTime+"ms");
				FBWCETPresentationDialog.DialogRunner presentationDialogRunner = new FBWCETPresentationDialog.DialogRunner(result);
				SwingUtilities.invokeAndWait(presentationDialogRunner);
			} catch (ReferencedTypeNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			
			
			/*
			LibraryElementPackage.eINSTANCE.eClass();
			
			ComposedAdapterFactory adapterFactory = new ComposedAdapterFactory();
			adapterFactory.addAdapterFactory(new LibraryElementAdapterFactory());
			
			BasicCommandStack commandStack = new BasicCommandStack();
			
			AdapterFactoryEditingDomain domain = new AdapterFactoryEditingDomain(adapterFactory, commandStack);
			
			Resource r = domain.getResourceSet().createResource(URI.createURI(selectedFile.getLocationURI().toString()));
			
			
			System.out.println(r.getURI());
			try {
				r.load(new HashMap<Object, Object>());
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			SystemManager.getInstance().loadSystems();
			List<AutomationSystem> systems = SystemManager.getInstance().getSystems();
			
			for (AutomationSystem system : systems) {
				for (PaletteEntry entry : system.getPalette().getFbGroup().getEntries()) {
					System.out.println(entry.getType());
				}
			}
			
			
			

			ResourceSet rs = SystemManager.getInstance().getResourceSet(selectedFile.getProject());
			//rs.getAdapterFactories().add(adapterFactory);
			Resource r2 = rs.createResource(URI.createURI(selectedFile.getLocationURI().toString()));
			try {
				r2.load(Collections.EMPTY_MAP);
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			
			TypeLibrary.getInstance();
			
			Resource r3 = new LibraryElementResourceFactoryImpl().createResource(URI.createURI(selectedFile.getLocationURI().toString()));
			try {
				r3.load(Collections.EMPTY_MAP);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			EList<EObject> list = SystemManager.getInstance().loadExternalModel(selectedFile.getProject(), URI.createURI(selectedFile.getLocationURI().toString()));
			System.out.println(list.size());
			
			
			System.out.println(r.toString());
			for (EObject eobject : r.getContents()) {
				System.out.println(eobject);
			}*/
			
		}
		
		
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		if (selection instanceof StructuredSelection) {
			StructuredSelection structuredSelection = (StructuredSelection) selection;
			Object selectedObject = structuredSelection.getFirstElement();
			if (selectedObject instanceof IFile) {
				this.selectedFile = (IFile) selectedObject;
				this.application = null;
				this.fbType = null;
			} else if (selectedObject instanceof Application) {
				this.selectedFile = null;
				this.application = (Application) selectedObject;
				this.fbType = null;
			} else if (selectedObject instanceof FBType) {
				this.selectedFile = null;
				this.application = null;
				this.fbType = (FBType) selectedObject;
			}
			
		}
	}
	
	class OptionsDialogRunner implements Runnable {
		private boolean resultOk;
		private WCETAnalysisOptions options;
				
		public OptionsDialogRunner(WCETAnalysisOptions options) {
			super();
			this.options = options;
		}

		@Override
		public void run() {
			
			try {
				UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			} catch (Exception e) {}
			
			WCETAnalysisOptionsEditorDialog dialog = new WCETAnalysisOptionsEditorDialog();
			dialog.setOptions(options);
			
			dialog.setModal(true);
			dialog.setVisible(true);
			options = dialog.getOptions();
			setResultOk(dialog.isResultOk());
						
		}

		public boolean isResultOk() {
			return resultOk;
		}

		protected void setResultOk(boolean resultOk) {
			this.resultOk = resultOk;
		}

		public WCETAnalysisOptions getOptions() {
			return options;
		}

		public void setOptions(WCETAnalysisOptions options) {
			this.options = options;
		}

		
		
		
		
	}

}
