package se.mdh.assist.analysis.wcet;

import org.fordiac.ide.model.libraryElement.BasicFBType;
import org.fordiac.ide.model.libraryElement.ECAction;
import org.fordiac.ide.model.libraryElement.ECC;
import org.fordiac.ide.model.libraryElement.ECState;
import org.fordiac.ide.model.libraryElement.ECTransition;
import org.fordiac.ide.model.libraryElement.Event;
import org.fordiac.ide.model.libraryElement.InterfaceList;

import se.mdh.assist.analysis.wcet.data.FBWCETData;
import se.mdh.assist.analysis.wcet.data.WCETAnalysisOptions;
import se.mdh.assist.analysis.wcet.data.WCETData;
import se.mdh.assist.analysis.wcet.data.WCETDataList;


public class BFBAnalysis extends AbstractWCETAnalysis {
	private BasicFBType bfb;
	
	public BFBAnalysis(BasicFBType bfb, WCETAnalysisOptions options) {
		super(options);
		this.bfb = bfb;
	}
	
	public FBWCETData doAnalysis() {
		InterfaceList interfaceList = bfb.getInterfaceList();
		
		FBWCETData fbWCETData = new FBWCETData();
		
		for (Event inputEvent : interfaceList.getEventInputs()) {
			ECC ecc = bfb.getECC();
			WCETDataList collectedData = new WCETDataList();
			for (ECTransition transition : ecc.getECTransition()) {
				if (transition.getConditionEvent() != null && transition.getConditionEvent().getName().equals(inputEvent.getName())) {
					collectedData.addAllWithNormalization(analyzeAllRuns(transition.getDestination()), options.normalizationMethod);
				}
			}
			fbWCETData.getEventData().put(inputEvent.getName(), collectedData);
		}
		
		return fbWCETData;
		
	}
	
	private WCETDataList analyzeAllRuns(ECState currentState) {
		WCETDataList result = new WCETDataList();
		WCETData currentStateData = new WCETData();
		for (ECAction action : currentState.getECAction()) {
			int wcet = -1;
			if (action.getAlgorithm() != null) {
				String comment = action.getAlgorithm().getComment();
				wcet = Helpers.getWCETFromCommentText(comment);
				if (wcet == -1) {
					System.err.println("No WCET definition for algorithm " + action.getAlgorithm().getName() + " of " + bfb.getName());
					continue;
				}	
			} else {
				wcet = 0;
			}
			String outputEventName = (action.getOutput() != null) ? action.getOutput().getName() : null;
			
			currentStateData.add(wcet, outputEventName);
		}
		for (ECTransition transition : currentState.getOutTransitions()) {
			if (transition.getConditionEvent() == null) {
				result.addAllWithNormalization(analyzeAllRuns(transition.getDestination()), options.normalizationMethod);
			}
		}
		if (result.size() == 0) {
			result.add(new WCETData());
		}
		for (WCETData data : result) {
			data.add(currentStateData);
		}
		return result;
	}
}
