package se.mdh.assist.analysis.wcet;

import java.util.ArrayList;
import java.util.Map;

import org.fordiac.ide.model.libraryElement.Event;
import org.fordiac.ide.model.libraryElement.EventConnection;
import org.fordiac.ide.model.libraryElement.FB;
import org.fordiac.ide.model.libraryElement.FBNetwork;
import org.fordiac.ide.model.libraryElement.InterfaceList;

import se.mdh.assist.analysis.wcet.data.FBWCETData;
import se.mdh.assist.analysis.wcet.data.OutputInfo;
import se.mdh.assist.analysis.wcet.data.PeriodWCETData;
import se.mdh.assist.analysis.wcet.data.WCETAnalysisOptions;
import se.mdh.assist.analysis.wcet.data.WCETData;
import se.mdh.assist.analysis.wcet.data.WCETDataList;

public abstract class FBNAnalysis extends AbstractWCETAnalysis {
	
	protected FBNetwork network;
	
	public FBNAnalysis(FBNetwork network, WCETAnalysisOptions options) {
		super(options);
		this.network = network;
	}
	
	protected WCETDataList doNetworkAnalysis(Event startEvent) {
		WCETDataList result = new WCETDataList();
		
		for (EventConnection connection : startEvent.getOutputConnections()) {
			Event targetEvent = connection.getDestination();
			if (targetEvent == null) {
				System.err.println("Error! No target event for connection '" + connection.getName() + "' from '" + startEvent.getName());
			}
			InterfaceList targetInterface = (InterfaceList) targetEvent.eContainer();
			

			// check if we've hit an output event of the enclosing composite
			/*if (targetInterface == this.cfb.getInterfaceList()) {
				OutputInfo newOutputInfo = new OutputInfo();
				newOutputInfo.addEvent(targetEvent.getName());
				result.add(new WCETData(0, newOutputInfo));
				continue;
			}*/
			if (!targetEvent.isIsInput()) {
				OutputInfo newOutputInfo = new OutputInfo();
				newOutputInfo.addEvent(targetEvent.getName());
				result.add(new WCETData(0, newOutputInfo));
				continue;
			}
			
			// pick up the WCET data for the target
			FB targetFB = (FB) targetInterface.eContainer();
			FBWCETData fbWcetData = Helpers.getFBWCETDataForFB(targetFB, options);
			
			if (fbWcetData == null) {
				System.err.println("NO wcet data for " + targetFB.getFBType().getName());
				return result;
			}
			
			WCETDataList wcetDataList = fbWcetData.getEventData(targetEvent.getName());
			
			if (wcetDataList == null) {
				System.err.println("NO wcet data for event input "+targetEvent.getName()+" of " + targetFB.getFBType().getName());
				return result;
			}
			
			// check all WCET data for the target FB
			WCETDataList resultForFB = new WCETDataList();
			for (WCETData targetData : wcetDataList) {
				// collect info for all output events, add it together...
				WCETDataList collectedDataList = collectRecursiveWCETData(targetData, targetInterface);
				
				// if still no collected data, add empty...
				if (collectedDataList.size() == 0) {
					collectedDataList.add(new WCETData());
				}
				// add wcet for currrrent targetData
				for (WCETData data : collectedDataList) {
					data.add(targetData.getWCETValue(), null);
				}
				resultForFB.addAllWithNormalization(collectedDataList, options.normalizationMethod);
			}
			result = WCETDataList.getAllCombinations(result, resultForFB);
		}
		return result;
	}
	
	protected ArrayList<PeriodWCETData> doPeriodAnalysis() {
		ArrayList<PeriodWCETData> result = new ArrayList<PeriodWCETData>();
		for (FB componentFB : network.getFBs()) {
			InterfaceList targetInterface = componentFB.getInterface();
			FBWCETData fbWCETData = Helpers.getFBWCETDataForFB(componentFB, options);
			if (fbWCETData == null) {
				System.err.println("No WCET data for "+ componentFB.getName()+ ", Type: " + componentFB.getFBType().getName()+"!");
				continue;
			}
			
			for (PeriodWCETData periodWCETData : fbWCETData.getPeriodData()) {
				WCETDataList resultForPeriod = new WCETDataList();
				for (WCETData wcetData : periodWCETData.getData()) {
					WCETDataList collectedDataList = collectRecursiveWCETData(wcetData, targetInterface);
					
					// if still no collected data, add empty...
					if (collectedDataList.size() == 0) {
						collectedDataList.add(new WCETData());
					}
					// add wcet for currrrent targetData
					for (WCETData data : collectedDataList) {
						data.add(wcetData.getWCETValue(), null);
					}
					resultForPeriod.addAllWithNormalization(collectedDataList, options.normalizationMethod);
				}	
				result.add(new PeriodWCETData(periodWCETData.getPeriod(), resultForPeriod));
			}
		}
		return result;
	}
	
	protected WCETDataList collectRecursiveWCETData(WCETData sourceData, InterfaceList targetInterface) {
		WCETDataList collectedDataList = new WCETDataList();
		
		Map<String, Integer> outputMap = sourceData.getOutputInfo().getMap();
		for (String outputEventName : outputMap.keySet()) {
			for (Event outputEvent : targetInterface.getEventOutputs()) {
				if(outputEventName.equals(outputEvent.getName())) {
					// get recursive results
					// TODO result caching? make deep clones?
					WCETDataList recursiveResults  = doNetworkAnalysis(outputEvent);
					recursiveResults.multiplyAll(outputMap.get(outputEventName));
					collectedDataList = WCETDataList.getAllCombinations(collectedDataList, recursiveResults);
				}
			}
		}
		return collectedDataList;
	}

	public FBNetwork getNetwork() {
		return network;
	}

	public void setNetwork(FBNetwork network) {
		this.network = network;
	}
}
