package se.mdh.assist.analysis.wcet;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.emf.ecore.xml.type.internal.DataValue.Base64;
import org.fordiac.ide.model.libraryElement.FB;
import org.fordiac.ide.model.libraryElement.FBType;
import org.fordiac.ide.model.libraryElement.ServiceInterfaceFBType;

import se.mdh.assist.analysis.wcet.data.FBWCETData;
import se.mdh.assist.analysis.wcet.data.WCETAnalysisOptions;
import se.mdh.assist.analysis.wcet.data.WCETAnalysisOptions.HierarchicalAnalysisType;



public class Helpers {
	public static int getWCETFromCommentText(String commentText) {
		if (commentText == null) {
			return -1;
		}
		
		Pattern wcetPattern = Pattern.compile("wcet=(.+?);");
		Matcher matcher = wcetPattern.matcher(commentText);
		if (matcher.find()) {
			String wcetString = matcher.group(1);
			try {
				return Integer.parseInt(wcetString);
			} catch (NumberFormatException nfe) {
				return -1;
			}
		} else {
			return -1;
		}
	}
	
	public static String getFB_WCETFromCommentText(String commentText) {
		Pattern wcetPattern = Pattern.compile("fb_wcet=(.+?);");
		Matcher matcher = wcetPattern.matcher(commentText);
		if (matcher.find()) {
			return matcher.group(1);
			
		} else {
			return null;
		}
	}
	
	public static String removeFB_WCETfromComment(String commentText) {
		Pattern wcetPattern = Pattern.compile("fb_wcet=(.+?);");
		Matcher matcher = wcetPattern.matcher(commentText);
		return matcher.replaceAll("");
	}
	
	public static String addFB_WCETtoComment(String commentText, String serializedFB_WCET) {
		return commentText + " fb_wcet=" + serializedFB_WCET + ";";
	}
	
	public static String setFB_WCETinComment(String commentText, String serializedFB_WCET) {
		String result = removeFB_WCETfromComment(commentText);
		result = addFB_WCETtoComment(result, serializedFB_WCET);
		return result;
		
	}
	
	public static String serializeToString(Serializable o) throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ObjectOutputStream oos = new ObjectOutputStream( baos );
        oos.writeObject( o );
        oos.close();
        return new String( Base64.encode( baos.toByteArray() ) );
	}
	
	public static FBWCETData deserializeFromString(String s ) throws IOException ,
		    ClassNotFoundException {
		if (s == null) {
			return null;
		}
		byte [] data = Base64.decode( s );
		ObjectInputStream ois = new ObjectInputStream( 
		new ByteArrayInputStream(  data ) );
		Object o  = ois.readObject();
		ois.close();
		return (FBWCETData) o;
		
	}
	
	public static FBWCETData deserializeFromCommentText(String commentText) throws IOException ,
    ClassNotFoundException {
		String text = getFB_WCETFromCommentText(commentText);
		return deserializeFromString(text);
	}
	
	public static FBWCETData getFBWCETDataForType(FBType fbType, WCETAnalysisOptions options) {
		try {
			FBWCETData fbWcetData = (FBWCETData) Helpers.deserializeFromString(
					Helpers.getFB_WCETFromCommentText(fbType.getComment()));
			
			if (options.hierarchicalAnalysisType == HierarchicalAnalysisType.FORCED
					|| (options.hierarchicalAnalysisType == HierarchicalAnalysisType.IF_NEEDED && fbWcetData == null)) {
				// we don't do analysis for SIFBs yet...
				if (!(fbType instanceof ServiceInterfaceFBType)) {
					WCETAnalysis hierarchicalAnalysis = new WCETAnalysis(fbType, options);
					fbWcetData = hierarchicalAnalysis.doAnalysis();
				}
			}
			
			return fbWcetData;
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}
	
	public static FBWCETData getFBWCETDataForFB(FB fb, WCETAnalysisOptions options) {
		FBType targetFBType = (FBType) fb.getPaletteEntry().getType();
		return getFBWCETDataForType(targetFBType, options);
	}
}
