package se.mdh.assist.analysis.wcet;

import java.io.IOException;

import org.fordiac.ide.model.libraryElement.Application;
import org.fordiac.ide.model.libraryElement.BasicFBType;
import org.fordiac.ide.model.libraryElement.CompositeFBType;
import org.fordiac.ide.model.libraryElement.FBType;
import org.fordiac.ide.model.libraryElement.I4DIACElement;

import se.mdh.assist.analysis.wcet.data.FBWCETData;
import se.mdh.assist.analysis.wcet.data.WCETAnalysisOptions;

public class WCETAnalysis implements IWCETAnalysis {
	
	I4DIACElement analysisTarget = null;
	WCETAnalysisOptions options;

	public WCETAnalysis(I4DIACElement analysisTarget, WCETAnalysisOptions options) {
		this.analysisTarget = analysisTarget;
		this.options = options;
	}
	
	@Override
	public FBWCETData doAnalysis() {
		FBWCETData result = null;
		FBType fbType = null;
		Application application = null;
		
		if (analysisTarget instanceof BasicFBType) {
			BasicFBType bfbt = (BasicFBType) analysisTarget;
			BFBAnalysis analysis = new BFBAnalysis(bfbt, options);
			result = analysis.doAnalysis();
			fbType = bfbt;
		} else if (analysisTarget instanceof CompositeFBType) {
			CompositeFBType cfbt = (CompositeFBType) analysisTarget;
			CFBAnalysis analysis = new CFBAnalysis(cfbt, options);
			result = analysis.doAnalysis();
			fbType = cfbt;
		} else if (analysisTarget instanceof Application) {
			application = (Application) analysisTarget;
			AppAnalysis analysis = new AppAnalysis(application, options);
			result = analysis.doAnalysis();
		} else {
			System.err.println("Unsuported FB type!");
		}
		
		if (result != null) {
			try {
				String ser = Helpers.serializeToString(result);
				if (fbType != null) {
					String newComment = Helpers.setFB_WCETinComment(fbType.getComment(), ser);
					fbType.setComment(newComment);
				} else if (application != null) {
					String newComment = Helpers.setFB_WCETinComment(application.getComment(), ser);
					application.setComment(newComment);
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		return result;
	}

	public WCETAnalysisOptions getOptions() {
		return options;
	}

	public void setOptions(WCETAnalysisOptions options) {
		this.options = options;
	}
	
	
}
