package se.mdh.assist.analysis.wcet.data;

import java.io.Serializable;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

public class OutputInfo implements Serializable, Cloneable {
	private static final long serialVersionUID = -6614479655221864289L;
	HashMap<String, Integer> outputMap;
			
	public OutputInfo() {
		super();
		outputMap = new HashMap<String, Integer>();
	}

	public void addEvent(String outputEventName) {
		addEvent(outputEventName, 1);
	}
	
	public void addEvent(String outputEventName, int number) {
		if (outputMap.containsKey(outputEventName)) {
			outputMap.put(outputEventName, outputMap.get(outputEventName) + number);
		} else {
			outputMap.put(outputEventName, number);
		}
	}
	
	public void add(OutputInfo other) {
		for (String event : other.outputMap.keySet()) {
			this.addEvent(event, other.outputMap.get(event));
		}
	}
	
	public void multiplyAll(int n) {
		for (String event : outputMap.keySet()) {
			outputMap.put(event, outputMap.get(event) * n);
		}
	}
	
	public static OutputInfo getSum(OutputInfo info1, OutputInfo info2) {
		OutputInfo result = new OutputInfo();
		result.add(info1);
		result.add(info2);
		return result;
	}
	
	public int getEventCount(String outputEventName) {
		if (outputMap.containsKey(outputEventName)) {
			return outputMap.get(outputEventName);
		} else {
			return 0;
		}
	}
	
	public Map<String, Integer> getMap() {
		return outputMap;
	}
	
	
	public CompareResult compareContent(OutputInfo target) {
		
		HashSet<String> events = new HashSet<String>();
		events.addAll(this.outputMap.keySet());
		events.addAll(target.outputMap.keySet());
		
		boolean thisWasSmaller = false;
		boolean targetWasSmaller = false;

		for (String event : events) {
			int thisNumber = (this.outputMap.get(event) != null) ? this.outputMap.get(event) : 0;
			int targetNumber = (target.outputMap.get(event) != null) ? target.outputMap.get(event) : 0;
			if (thisNumber < targetNumber) {
				thisWasSmaller = true;
			} else if (thisNumber > targetNumber) {
				targetWasSmaller = true;
			}
			if (thisWasSmaller && targetWasSmaller) {
				return CompareResult.UNCOMPARABLE;
			}
		}		
		
		if (thisWasSmaller) {
			return CompareResult.SMALLER;
		} else if (targetWasSmaller) {
			return CompareResult.BIGGER;
		}
		return CompareResult.SAME;
	}

	@Override
	public String toString() {
		/*StringBuilder sb = new StringBuilder();
		
		sb.append("[");
		for (String outputEvent : outputMap.keySet()) {
			sb.append("<");
			sb.append(outputEvent);
			sb.append(", ");
			sb.append(outputMap.get(outputEvent));
			sb.append(">");
		}
		sb.append("]");
		return sb.toString();*/
		return outputMap.toString();
	}
	
	public static enum CompareResult {
		BIGGER,
		SMALLER,
		SAME,
		UNCOMPARABLE
	}

	@SuppressWarnings("unchecked")
	@Override
	protected Object clone() throws CloneNotSupportedException {
		OutputInfo result = new OutputInfo();
		result.outputMap = (HashMap<String, Integer>) outputMap.clone();
		
		return result;
	}
	
	
	
}