package se.mdh.assist.analysis.wcet.data;

import java.io.Serializable;

import se.mdh.assist.analysis.wcet.data.OutputInfo.CompareResult;

public class WCETData implements Serializable, Comparable<WCETData>, Cloneable {
	private static final long serialVersionUID = -2962976735258402205L;
	private int wcetValue;
	private OutputInfo outputInfo;
	
	public WCETData() {
		super();
		wcetValue = 0;
		outputInfo = new OutputInfo();
	}
	
	public WCETData(int wcetValue, String outputEventName) {
		this();
		this.wcetValue = wcetValue;
		outputInfo.addEvent(outputEventName);
	}
	
	public WCETData(int wcetValue, OutputInfo outputInfo) {
		this();
		this.wcetValue = wcetValue;
		this.outputInfo = outputInfo;
	}

	
	
	public void add(int wcetValue, String outputEventName) {
		this.wcetValue += wcetValue;
		if (outputEventName != null) {
			outputInfo.addEvent(outputEventName);
		}
	}
	
	public int getWCETValue() {
		return wcetValue;
	}
	
	public void addWCET(int wcetValue) {
		this.wcetValue += wcetValue;
		
	}
	
	public void setWCETValue(int wcetValue) {
		this.wcetValue = wcetValue;
	}
	public OutputInfo getOutputInfo() {
		return outputInfo;
	}
	
	
	
	
	public void setOutputInfo(OutputInfo outputInfo) {
		this.outputInfo = outputInfo;
	}

	@Override
	public int compareTo(WCETData target) {
		int wcetResult = this.wcetValue - target.wcetValue;
		OutputInfo.CompareResult outputResult = this.outputInfo.compareContent(target.outputInfo);
		
		if (wcetResult <= 0 && (outputResult == CompareResult.SMALLER || outputResult == CompareResult.SAME)) {
			return -1;
		} else if (wcetResult >= 0 && (outputResult == CompareResult.BIGGER || outputResult == CompareResult.SAME)) {
			return 1;
		}
			
		return 0;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("<");
		sb.append(wcetValue);
		sb.append(", ");
		sb.append(outputInfo.toString());
		sb.append(">");
		return sb.toString();
	}

	@Override
	protected Object clone() throws CloneNotSupportedException {
		WCETData result = new WCETData();
		result.outputInfo = (OutputInfo) this.outputInfo.clone();
		result.wcetValue = this.wcetValue;
		return result;
	}

	public void multiplyWcet(int n) {
		wcetValue *= n;
		
	}

	public void add(WCETData other) {
		this.wcetValue += other.wcetValue;
		this.outputInfo.add(other.outputInfo);
	}
}