package se.mdh.assist.analysis.wcet.data;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;

import se.mdh.assist.analysis.wcet.data.WCETAnalysisOptions.NormalizationMethod;

public class WCETDataList extends ArrayList<WCETData> {
	private static final long serialVersionUID = -3366929987724712706L;
	
	public void addWithNormalization(WCETData newData, NormalizationMethod method) {
		
		if (method == NormalizationMethod.MAXIMAL_ELEMENTS) { 
			
			addWithMaximalElementsNormalization(newData);
		} else if (method == NormalizationMethod.SUPREMUM) {
			addWithSupremumNormalization(newData);
		}
		
		
	}
	
	public void addWithMaximalElementsNormalization(WCETData newData) {
		for(int i=0; i < this.size(); i++) {
			int compareResult = newData.compareTo(this.get(i)); 
			if (compareResult < 0) {
				//no need to add this, there are wider entries...
				return;
			} else if (compareResult > 0) {
				// this entry is wider than existing, remove the existing...
				this.remove(i);
				i--;
			}
		}
		this.add(newData);
	}
	
	public void addWithSupremumNormalization(WCETData data) {
		if (this.size() == 0) {
			this.add(data);
			return;
		}
		for (WCETData existingData : this) {
			int newWCETValue = (data.getWCETValue() > existingData.getWCETValue()) ? data.getWCETValue() : existingData.getWCETValue();
			existingData.setWCETValue(newWCETValue);
			
			HashSet<String> newKeys = new HashSet<String>();
			newKeys.addAll(data.getOutputInfo().getMap().keySet());
			newKeys.addAll(existingData.getOutputInfo().getMap().keySet());
			
			OutputInfo newOutputInfo = new OutputInfo();
			for (String event : newKeys) {
				newOutputInfo.getMap().put(event, (data.getOutputInfo().getEventCount(event) > existingData.getOutputInfo().getEventCount(event))?
						data.getOutputInfo().getEventCount(event) :
							existingData.getOutputInfo().getEventCount(event)); 
			}
			existingData.setOutputInfo(newOutputInfo);
		}
	}
	
	
	public void addAllWithNormalization(Collection<WCETData> newDataCollection, NormalizationMethod method) {
		for (WCETData newData : newDataCollection) {
			addWithNormalization(newData, method);
		}
	}
	
	public void multiplyAll(int n) {
		for (WCETData data : this) {
			data.multiplyWcet(n);
			data.getOutputInfo().multiplyAll(n);
		}
	}
	
	public static WCETDataList getAllCombinations(WCETDataList list1, WCETDataList list2) {
		WCETDataList result = new WCETDataList();
		if (list1.size() == 0) {
			list1.add(new WCETData());
		}
		if (list2.size() == 0) {
			list2.add(new WCETData());
		}
		
		for (WCETData data1 : list1) {
			for (WCETData data2 : list2) {
				int resultWCET = data1.getWCETValue() + data2.getWCETValue();
				OutputInfo resultOutputInfo = OutputInfo.getSum(data1.getOutputInfo(), data2.getOutputInfo());
				result.add(new WCETData(resultWCET, resultOutputInfo));
			}
		}
		
		return result;
	}
	
}
