package se.mdh.assist.analysis.wcet.gui;

import java.awt.Component;
import java.util.ArrayList;

import javax.swing.AbstractCellEditor;

import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;

import se.mdh.assist.analysis.wcet.data.PeriodWCETData;

public class PeriodList extends JTable {
	private static final long serialVersionUID = -708990043796712309L;
	
	private DefaultTableModel model;
	private ArrayList<PeriodWCETData> data;
	
	public PeriodList() {
		super(new DefaultTableModel());
		model = (DefaultTableModel) this.getModel();
		
		model.addColumn("Period");
		this.setTableHeader(null);
		getColumnModel().getColumn(0).setCellRenderer(new PeriodListCellRenderer());
		getColumnModel().getColumn(0).setCellEditor(new PeriodListCellEditor());
	}

	public ArrayList<PeriodWCETData> getData() {
		return data;
	}

	public void setData(ArrayList<PeriodWCETData> data) {
		if (getCellEditor() != null) {
			getCellEditor().stopCellEditing();
		}
		this.data = data;
		update();
		clearSelection();
	}
	
	public void update() {
		
		while (model.getRowCount() > 0) {
			model.removeRow(0);
		}
		if (data != null) {
			for (PeriodWCETData periodData : data) {
				Object[] row = {periodData};
				model.addRow(row);
			}
		} else {
		}
	}
	
	private class PeriodListCellEditor extends AbstractCellEditor implements TableCellEditor {
		private static final long serialVersionUID = 163213774327886400L;
		JTextField field;
		PeriodWCETData periodData;		

		public PeriodListCellEditor() {
			super();
			field = new JTextField();
			periodData = null;
		}

		@Override
		public Object getCellEditorValue() {
			if (periodData != null) {
				try {
					periodData.setPeriod(Integer.parseInt(field.getText()));
				} catch (Exception e) {
				}
			}
			return periodData;
		}

		@Override
		public Component getTableCellEditorComponent(JTable table,
				Object value, boolean isSelected, int row, int column) {
			if (value instanceof PeriodWCETData) {
				PeriodWCETData periodValue = (PeriodWCETData) value;
				field.setText(Integer.toString(periodValue.getPeriod()));
				periodData = periodValue;
			} else {
				periodData = null;
				field.setText("");
			}
			return field;
		}
	}
	
	private class PeriodListCellRenderer extends DefaultTableCellRenderer {
		private static final long serialVersionUID = 4585378163464197587L;

		@Override
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
					row, column);
			if (c instanceof PeriodListCellRenderer) {
				PeriodListCellRenderer renderer = (PeriodListCellRenderer) c;
				if (value instanceof PeriodWCETData) {
					PeriodWCETData periodData = (PeriodWCETData) value;
					renderer.setText(Integer.toString(periodData.getPeriod()));
					renderer.setToolTipText(periodData.getOriginStack().toString());
				}
			}
			
			return c;
		}
		
	}
}
