package se.mdh.assist.analysis.wcet.gui;

import javax.swing.JPanel;

import se.mdh.assist.analysis.wcet.data.WCETAnalysisOptions;
import se.mdh.assist.analysis.wcet.data.WCETAnalysisOptions.HierarchicalAnalysisType;
import se.mdh.assist.analysis.wcet.data.WCETAnalysisOptions.NormalizationMethod;

import java.awt.BorderLayout;
import javax.swing.JSplitPane;
import javax.swing.border.TitledBorder;
import javax.swing.JRadioButton;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import javax.swing.ButtonGroup;

public class WCETAnalysisOptionsEditPanel extends JPanel {
	private static final long serialVersionUID = -8844975532934027523L;
	private JSplitPane splitPane;
	private JPanel normalizationMethodPanel;
	private JPanel hierarchicalAnalyisTypePanel;
	private JRadioButton rdbtnMaximalElements;
	private JRadioButton rdbtnSupremum;
	private JRadioButton rdbtnNoHiererchicalAnalysis;
	private JRadioButton rdbtnIfNeeded;
	private JRadioButton rdbtnForced;
	private final ButtonGroup normalizationMethodButtons = new ButtonGroup();
	private final ButtonGroup hierarchicalAnalysisTypeButtons = new ButtonGroup();
	
	/**
	 * Create the panel.
	 */
	public WCETAnalysisOptionsEditPanel() {
		setLayout(new BorderLayout(0, 0));
		add(getSplitPane(), BorderLayout.CENTER);
	}

	public WCETAnalysisOptions getOptions() {
		WCETAnalysisOptions options = new WCETAnalysisOptions();
		
		if (getRdbtnMaximalElements().isSelected()) {
			options.normalizationMethod = NormalizationMethod.MAXIMAL_ELEMENTS;
		} else if (getRdbtnSupremum().isSelected()) {
			options.normalizationMethod = NormalizationMethod.SUPREMUM;
		}
		
		if (getRdbtnNoHiererchicalAnalysis().isSelected()) {
			options.hierarchicalAnalysisType = HierarchicalAnalysisType.NON_HIERARCHICAL;
		} else if (getRdbtnIfNeeded().isSelected()) {
			options.hierarchicalAnalysisType = HierarchicalAnalysisType.IF_NEEDED;
		} else if (getRdbtnForced().isSelected()) {
			options.hierarchicalAnalysisType = HierarchicalAnalysisType.FORCED;
		}
		
		return options;
	}

	public void setOptions(WCETAnalysisOptions options) {
		switch(options.normalizationMethod) {
		case MAXIMAL_ELEMENTS:
			getRdbtnMaximalElements().setSelected(true);
			break;
		case SUPREMUM:
			getRdbtnSupremum().setSelected(true);
			break;
		}
		
		switch (options.hierarchicalAnalysisType) {
		case NON_HIERARCHICAL:
			getRdbtnNoHiererchicalAnalysis().setSelected(true);
			break;
		case IF_NEEDED:
			getRdbtnIfNeeded().setSelected(true);
			break;
		case FORCED:
			getRdbtnForced().setSelected(true);
			break;
		}
	}

	
	
	private JSplitPane getSplitPane() {
		if (splitPane == null) {
			splitPane = new JSplitPane();
			splitPane.setLeftComponent(getNormalizationMethodPanel());
			splitPane.setRightComponent(getHierarchicalAnalyisTypePanel());
		}
		return splitPane;
	}
	private JPanel getNormalizationMethodPanel() {
		if (normalizationMethodPanel == null) {
			normalizationMethodPanel = new JPanel();
			normalizationMethodPanel.setBorder(new TitledBorder(null, "Normalization method", TitledBorder.LEADING, TitledBorder.TOP, null, null));
			GridBagLayout gbl_normalizationMethodPanel = new GridBagLayout();
			gbl_normalizationMethodPanel.columnWidths = new int[]{55, 69, 0};
			gbl_normalizationMethodPanel.rowHeights = new int[]{23, 0, 0};
			gbl_normalizationMethodPanel.columnWeights = new double[]{0.0, 0.0, Double.MIN_VALUE};
			gbl_normalizationMethodPanel.rowWeights = new double[]{0.0, 0.0, Double.MIN_VALUE};
			normalizationMethodPanel.setLayout(gbl_normalizationMethodPanel);
			GridBagConstraints gbc_rdbtnWiden = new GridBagConstraints();
			gbc_rdbtnWiden.anchor = GridBagConstraints.NORTHWEST;
			gbc_rdbtnWiden.insets = new Insets(0, 0, 5, 5);
			gbc_rdbtnWiden.gridx = 0;
			gbc_rdbtnWiden.gridy = 0;
			normalizationMethodPanel.add(getRdbtnMaximalElements(), gbc_rdbtnWiden);
			GridBagConstraints gbc_rdbtnMaximize = new GridBagConstraints();
			gbc_rdbtnMaximize.insets = new Insets(0, 0, 0, 5);
			gbc_rdbtnMaximize.anchor = GridBagConstraints.NORTHWEST;
			gbc_rdbtnMaximize.gridx = 0;
			gbc_rdbtnMaximize.gridy = 1;
			normalizationMethodPanel.add(getRdbtnSupremum(), gbc_rdbtnMaximize);
		}
		return normalizationMethodPanel;
	}
	private JPanel getHierarchicalAnalyisTypePanel() {
		if (hierarchicalAnalyisTypePanel == null) {
			hierarchicalAnalyisTypePanel = new JPanel();
			hierarchicalAnalyisTypePanel.setBorder(new TitledBorder(null, "Hierarchical analysis type", TitledBorder.LEADING, TitledBorder.TOP, null, null));
			GridBagLayout gbl_hierarchicalAnalyisTypePanel = new GridBagLayout();
			gbl_hierarchicalAnalyisTypePanel.columnWidths = new int[]{36, 137, 73, 0};
			gbl_hierarchicalAnalyisTypePanel.rowHeights = new int[]{23, 0, 23, 0};
			gbl_hierarchicalAnalyisTypePanel.columnWeights = new double[]{0.0, 0.0, 0.0, Double.MIN_VALUE};
			gbl_hierarchicalAnalyisTypePanel.rowWeights = new double[]{0.0, 0.0, 0.0, Double.MIN_VALUE};
			hierarchicalAnalyisTypePanel.setLayout(gbl_hierarchicalAnalyisTypePanel);
			GridBagConstraints gbc_rdbtnNoHiererchicalAnalysis = new GridBagConstraints();
			gbc_rdbtnNoHiererchicalAnalysis.anchor = GridBagConstraints.NORTHWEST;
			gbc_rdbtnNoHiererchicalAnalysis.insets = new Insets(0, 0, 5, 5);
			gbc_rdbtnNoHiererchicalAnalysis.gridx = 1;
			gbc_rdbtnNoHiererchicalAnalysis.gridy = 0;
			hierarchicalAnalyisTypePanel.add(getRdbtnNoHiererchicalAnalysis(), gbc_rdbtnNoHiererchicalAnalysis);
			GridBagConstraints gbc_rdbtnIfNeeded = new GridBagConstraints();
			gbc_rdbtnIfNeeded.anchor = GridBagConstraints.NORTHWEST;
			gbc_rdbtnIfNeeded.insets = new Insets(0, 0, 5, 5);
			gbc_rdbtnIfNeeded.gridx = 1;
			gbc_rdbtnIfNeeded.gridy = 1;
			hierarchicalAnalyisTypePanel.add(getRdbtnIfNeeded(), gbc_rdbtnIfNeeded);
			GridBagConstraints gbc_rdbtnForced = new GridBagConstraints();
			gbc_rdbtnForced.anchor = GridBagConstraints.NORTHWEST;
			gbc_rdbtnForced.insets = new Insets(0, 0, 0, 5);
			gbc_rdbtnForced.gridx = 1;
			gbc_rdbtnForced.gridy = 2;
			hierarchicalAnalyisTypePanel.add(getRdbtnForced(), gbc_rdbtnForced);
		}
		return hierarchicalAnalyisTypePanel;
	}
	private JRadioButton getRdbtnMaximalElements() {
		if (rdbtnMaximalElements == null) {
			rdbtnMaximalElements = new JRadioButton("Maximal elements");
			normalizationMethodButtons.add(rdbtnMaximalElements);
		}
		return rdbtnMaximalElements;
	}
	private JRadioButton getRdbtnSupremum() {
		if (rdbtnSupremum == null) {
			rdbtnSupremum = new JRadioButton("Supremum");
			normalizationMethodButtons.add(rdbtnSupremum);
		}
		return rdbtnSupremum;
	}
	private JRadioButton getRdbtnNoHiererchicalAnalysis() {
		if (rdbtnNoHiererchicalAnalysis == null) {
			rdbtnNoHiererchicalAnalysis = new JRadioButton("No hiererchical analysis");
			hierarchicalAnalysisTypeButtons.add(rdbtnNoHiererchicalAnalysis);
		}
		return rdbtnNoHiererchicalAnalysis;
	}
	private JRadioButton getRdbtnIfNeeded() {
		if (rdbtnIfNeeded == null) {
			rdbtnIfNeeded = new JRadioButton("If needed");
			hierarchicalAnalysisTypeButtons.add(rdbtnIfNeeded);
		}
		return rdbtnIfNeeded;
	}
	private JRadioButton getRdbtnForced() {
		if (rdbtnForced == null) {
			rdbtnForced = new JRadioButton("Forced");
			hierarchicalAnalysisTypeButtons.add(rdbtnForced);
		}
		return rdbtnForced;
	}
}
