package se.mdh.assist.analysis.wcet.gui;

import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

import se.mdh.assist.analysis.wcet.data.FBWCETData;
import se.mdh.assist.analysis.wcet.data.PeriodWCETData;
import se.mdh.assist.analysis.wcet.data.WCETData;
import se.mdh.assist.analysis.wcet.data.WCETDataList;
import se.mdh.assist.analysis.wcet.data.PeriodWCETData.PeriodOrigin;

public class WCETDataPresentationTable extends JTable {
	private static final String INPUT_EVENT_PERIOD_COLUMN = "Input event / Period";
	private static final String WCET_VALUE_COLUMN = "WCET value";
	private static final String OUTPUT_EVENT_COLUMN = "Output event";
	private static final String N_COLUMN = "N";
	private static final long serialVersionUID = 5057946743316918501L;
	private FBWCETData data;
	private DefaultTableModel model;
	

	public WCETDataPresentationTable() {
		super();
		model = new DefaultTableModel();
		
		model.addColumn(INPUT_EVENT_PERIOD_COLUMN);
		model.addColumn(WCET_VALUE_COLUMN);
		model.addColumn(OUTPUT_EVENT_COLUMN);
		model.addColumn(N_COLUMN);
		
		this.setModel(model);
		this.getColumn(INPUT_EVENT_PERIOD_COLUMN).setCellRenderer(new WCETDataCellRenderer());
		this.getColumn(WCET_VALUE_COLUMN).setCellRenderer(new WCETDataCellRenderer());
		this.getColumn(OUTPUT_EVENT_COLUMN).setCellRenderer(new WCETDataCellRenderer());
		this.getColumn(N_COLUMN).setCellRenderer(new WCETDataCellRenderer());
		
	}

	public FBWCETData getData() {
		return data;
	}

	public void setData(FBWCETData data) {
		this.data = data;
		updateModel();
	}
	
	private void updateModel() {
		while (model.getRowCount() > 0) {
			model.removeRow(0);
		}
		
		HashMap<String, WCETDataList> eventData = data.getEventData();
		for (String inputEvent : eventData.keySet()) {
			WCETDataList wcetDataList = eventData.get(inputEvent);
			boolean firstForInputEvent = true;
			
			for (WCETData wcetData : wcetDataList) {
							
				boolean firstForWcetData = true;
				
				Map<String, Integer> outputInfoMap = wcetData.getOutputInfo().getMap();
				
				if (outputInfoMap.size() == 0) {
					Object[] row = new Object[4];
					if (firstForInputEvent) {
						row[0] = inputEvent;
						firstForInputEvent = false;
					}
					if (firstForWcetData) {
						row[1] = wcetData.getWCETValue();
						firstForWcetData = false;
					}
					model.addRow(row);
				}
				
				for (String outputEvent : outputInfoMap.keySet()) {
					Object[] row = new Object[4];
					if (firstForInputEvent) {
						row[0] = inputEvent;
						firstForInputEvent = false;
					}
					if (firstForWcetData) {
						row[1] = wcetData.getWCETValue();
						firstForWcetData = false;
					}
					row[2] = outputEvent;
					row[3] = outputInfoMap.get(outputEvent);
					model.addRow(row);
				}
			}
			
		}
		
		for (PeriodWCETData periodData : data.getPeriodData()) {
			boolean firstForPeriodData = true;
			
			for (WCETData wcetData : periodData.getData()) {
				boolean firstForWcetData = true;
				
				Map<String, Integer> outputInfoMap = wcetData.getOutputInfo().getMap();
				
				if (outputInfoMap.size() == 0) {
					Object[] row = new Object[4];
					if (firstForPeriodData) {
						CellPeriodData cellPeriodData = new CellPeriodData();
						cellPeriodData.period = periodData.getPeriod();
						cellPeriodData.originStack = periodData.getOriginStack();
						row[0] = cellPeriodData;
						firstForPeriodData = false;
					}
					if (firstForWcetData) {
						row[1] = wcetData.getWCETValue();
						firstForWcetData = false;
					}
					model.addRow(row);
				}
				
				for (String outputEvent : outputInfoMap.keySet()) {
					Object[] row = new Object[4];
					if (firstForPeriodData) {
						CellPeriodData cellPeriodData = new CellPeriodData();
						cellPeriodData.period = periodData.getPeriod();
						cellPeriodData.originStack = periodData.getOriginStack();
						row[0] = cellPeriodData;
						firstForPeriodData = false;
					}
					if (firstForWcetData) {
						row[1] = wcetData.getWCETValue();
						firstForWcetData = false;
					}
					row[2] = outputEvent;
					row[3] = outputInfoMap.get(outputEvent);
					model.addRow(row);
				}
			}
		}
	}

	private class WCETDataCellRenderer extends DefaultTableCellRenderer {
		private static final long serialVersionUID = -297725765623501162L;

		@Override
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
					row, column);
			
			if (c instanceof JLabel && !hasFocus) {
				JLabel label = (JLabel) c;
				label.setBackground(Color.WHITE);
				if (column == 0 && value != null && value != "") {
					label.setBackground(Color.LIGHT_GRAY);
				}
				if (column == 1 && value != null && value != "") {
					label.setBackground(Color.LIGHT_GRAY);
				}
				if (column == 2 && value != null && value != "") {
					label.setBackground(Color.LIGHT_GRAY);
				}
				if (column == 3 && value != null && value != "") {
					label.setBackground(Color.LIGHT_GRAY);
				}
				
				if (value instanceof CellPeriodData) {
					CellPeriodData cellPeriodData = (CellPeriodData) value;
					label.setText(Integer.toString(cellPeriodData.period));
					label.setToolTipText(cellPeriodData.originStack.toString());					

				}
			}
			
						
			return c;
		}
		
	}
	
	private static class CellPeriodData {
		public int period;
		public ArrayList<PeriodOrigin> originStack;

		@Override
		public String toString() {
			return Integer.toString(period);
		}
		
		
	}
}
