package se.mdh.assist.analysis.wcet.gui;

import java.awt.Component;

import javax.swing.AbstractCellEditor;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;

import se.mdh.assist.analysis.wcet.data.WCETData;
import se.mdh.assist.analysis.wcet.data.WCETDataList;

public class WCETValueList extends JTable {
	private static final long serialVersionUID = -6923380812213316445L;
	private DefaultTableModel model;
	private WCETDataList data = null;
	private TableColumn wcetColumn;
	
	public WCETValueList() {
		super(new DefaultTableModel());
		this.model = (DefaultTableModel) this.getModel();
		//this.setCellRenderer(new WCETValueListCellRenderer());
		
		model.addColumn("Output event");
		this.setTableHeader(null);
		getColumnModel().getColumn(0).setCellRenderer(new WCETValueListCellRenderer());
		getColumnModel().getColumn(0).setCellEditor(new WCETValueListCellEditor());
	}
	
	public TableColumn getWcetColumn() {
		if (wcetColumn == null ){
			wcetColumn = new TableColumn();
			wcetColumn.setMinWidth(50);
			wcetColumn.setCellRenderer(new WCETValueListCellRenderer());
		}
		return wcetColumn;
	}
	
	public WCETDataList getData() {
		return data;
	}

	public void setData(WCETDataList data) {
		if (getCellEditor() != null) {
			getCellEditor().stopCellEditing();
		}
		this.data = data;
		update();
	}
	
	public void update() {
		while (model.getRowCount() > 0) {
			model.removeRow(0);
		}
		if (data != null) {
			for (WCETData dataElement : data) {
				Object[] row = {dataElement};
				model.addRow(row);
			}
		} else {
		}
	}
	
	private class WCETValueListCellEditor extends AbstractCellEditor implements TableCellEditor {
		private static final long serialVersionUID = 163213774327886400L;
		JTextField field;
		WCETData wcetData;
		

		public WCETValueListCellEditor() {
			super();
			field = new JTextField();
		}

		@Override
		public Object getCellEditorValue() {
			if (wcetData != null) {
				try {
					wcetData.setWCETValue(Integer.parseInt(field.getText()));
				} catch (Exception e) {
				}
			}
			return wcetData;
		}

		@Override
		public Component getTableCellEditorComponent(JTable table,
				Object value, boolean isSelected, int row, int column) {
			if (value instanceof WCETData) {
				wcetData = (WCETData) value;
				field.setText(Integer.toString(wcetData.getWCETValue()));
			} else {
				wcetData = null;
				field.setText("");
			}
			return field;
		}
		
		
		
		
		
	}


	private class WCETValueListCellRenderer extends DefaultTableCellRenderer {
		private static final long serialVersionUID = 4585378163464197587L;

		@Override
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
					row, column);
			if (value instanceof WCETData) {
				WCETData wcetData = (WCETData) value;
				if (c instanceof WCETValueListCellRenderer) {
					WCETValueListCellRenderer renderer = (WCETValueListCellRenderer) c;
					renderer.setText(Integer.toString(wcetData.getWCETValue()));
				}
				
			}
			
			return c;
		}
		
	}
}
